/*******************************************************************************************************************************************


		Compton Camera image generator software

			version:
				0.20 beta
			date:
				8 November 2006
			author:
				Karoly Nagy
				Szilard Szalay


*******************************************************************************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define PI 3.1415
#define MAXEFELB 100
#define AVOGADRO 6.022E+23
#define AXIAL 1
#define	POLAR 2
#define DESCARTES 0

#define POINTS 0
#define LINE 1
#define RING 2
#define NTI 4
#define SI 1
#define NAI 3
#define TWOPI 6.283
#define MAXSOURCEDATA 30


/*******  type definitions  ***************************************************************************************************************/


typedef struct digifoto {						/* this structure contains the data of fotons (element of a list) */
	long int index;							/*     have come on this specific path */
	long int n;							/*     n pieces */
	int multiscatter;
	struct digifoto *next;						/* pointer to the next element of this list */
} tdigifoto;


typedef struct {							/* the same as above, but this is an element of a vector */
	long int index;
	long int n;
	int multiscatter;
	int nonzeroprob;						/* TRUE, if "returnofjimi" give nonzero probability for this path */
} tdfvect;


typedef struct data {							/* All important data in this structure */
	char infname[255];						/* name of the inputfile */
	char xslibname[255];

	int sourcemode, numpoints;
	long int activity, ejj, backgract;
	double sourceloc[MAXSOURCEDATA];
	int sourcedatanum;
	double xsec[5][142];
	int scatdetmat, absdetmat;
	int scatzmode, onescatmode, twoscatmode, abszmode, doppler;
	int posscatnoise, posabsnoise, energynoise, background;
	double posscatfwhm, posabsfwhm, energyfwhm;
	double scatzdist, abszdist;
	double dopplerdata[9];
	
	tdigifoto *dflisthead;						/* headpointer of digitalized foton data list */
	tdigifoto *dfpointer;						/* pinter on digitalized foton data list */
	long int digifotonum;						/* number of d.f. */
	
	double scatdetxmin, scatdetxmax, scatdetymin, scatdetymax;
	double absdetxmin, absdetxmax, absdetymin, absdetymax;
		
	tdfvect *dfvect;
	
	double mx, my, dx, dy, dist1, ux, uy, dist2;			/* geometrical data */
	double pixm, pixd, pixu, thick, thick2;
	double efelb;							/* energy data */
	double energy;
	double rho, M, Z;						/* matter of scatter detector */

	double *colsum;							/* pointer forcolsum-vector */
	double **probmx;						/* pointre for probability matrix*/
	long int pmxrownum, pmxcolnum;					/* numbers of rows and columns of pmatrix */

	double *lambda[2];						/* pointer for the data of steps of iteration */
	int flipflop;							/* index says whitch lambda[i] does contain 
									   the result of last iteration*/
	int maxiter;							/* number of iteration */
	int printiter;
} tdata;


typedef struct {
	int firstline[7];
	double secondline[9];
} tinputrecord;



/*******  initialization routines  ********************************************************************************************************/


void init ( tdata *calcdata ) {	
	calcdata->infname[0]='\0';
	calcdata->xslibname[0]='\0';
	
	calcdata->sourcemode = POINTS;
	calcdata->sourcedatanum = 0;
	calcdata->scatdetmat = SI;
	calcdata->absdetmat = NAI;
	calcdata->scatzmode = 0;
	calcdata->onescatmode = 0;
	calcdata->twoscatmode = 0;
	calcdata->abszmode = 0;
	calcdata->doppler = 0;
	calcdata->activity = 1000000;
	calcdata->posscatnoise = 0;
	calcdata->posabsnoise = 0; 
	calcdata->energynoise = 0;
	calcdata->background = 0;
	calcdata->backgract = 0;
	calcdata->ejj = 0;
	
	calcdata->dflisthead = NULL;
	calcdata->dfpointer = NULL;
	calcdata->digifotonum = 0;

	calcdata->dfvect = NULL;

	calcdata->probmx = NULL;
	calcdata->colsum = NULL;
	calcdata->lambda[0] = NULL;
	calcdata->lambda[1] = NULL;
	calcdata->flipflop = 0;

	calcdata->maxiter = 100;
	calcdata->printiter = 5;
}

void setgeomdata( tdata *calcdata ) {
	calcdata->numpoints=(calcdata->sourcedatanum)/3.0; /*!!*/
	calcdata->absdetxmin=-calcdata->ux/2.0;
	calcdata->absdetxmax=calcdata->ux/2.0;
	calcdata->absdetymin=-calcdata->uy/2.0;
	calcdata->absdetymax=calcdata->uy /2.0;
	calcdata->scatdetxmax=calcdata->dx/2.0;
	calcdata->scatdetxmin=-calcdata->dx/2.0;
	calcdata->scatdetymax=calcdata->dy/2.0;
	calcdata->scatdetymin=-calcdata->dy/2.0;
}	

int initdfvect( tdata *calcdata ) {
	if( !(calcdata->dfvect = (tdfvect*)malloc(calcdata->digifotonum*sizeof(tdfvect)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 101;
	}
	return 0;
}


int initpm ( tdata *calcdata ) {
	long int row,col;

	calcdata->pmxcolnum = round(calcdata->mx * calcdata->my / pow(calcdata->pixm, 2));
	calcdata->pmxrownum = round( calcdata->ux * calcdata->uy / pow(calcdata->pixu, 2) * 
				 calcdata->dx * calcdata->dy / pow(calcdata->pixd, 2) *
				 100.0/calcdata->efelb);
	
	if( !(calcdata->colsum = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 121 ;
	}

	if( !(calcdata->probmx = (double**)malloc(calcdata->digifotonum*sizeof(double*)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 122;
	}

	for( row = 0; row<calcdata->digifotonum; row++ ) {
		if( !(calcdata->probmx[row] = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 123;
		}
	}
	
	for( row = 0; row < calcdata->digifotonum; row++ ) {
	        for( col = 0; col < calcdata->pmxcolnum; col++ ) {
			calcdata->probmx[row][col] = 0;
		}
	}
	
	return 0;
}


int initlambda ( tdata * calcdata ) {
	long int l;
	int li;

	for( li=0; li<2; li++ ) {
		if( !(calcdata->lambda[li] = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 131;
		}
		for( l=0; l<calcdata->pmxcolnum; l++ ) {
			calcdata->lambda[li][l] = 1;
		}
	}
	return 0;
}


/*******  freeing unused memory  **********************************************************************************************************/


void freedflist( tdata *calcdata ) {
	tdigifoto *kukker;
	
	calcdata->dfpointer = calcdata->dflisthead;
	calcdata->dflisthead = NULL;
	while( calcdata->dfpointer ) {
		kukker = calcdata->dfpointer;
		calcdata->dfpointer = calcdata->dfpointer->next;
		free(kukker);
	}
}


/*******  reading inputfile  **************************************************************************************************************/


void cutcomm( char *inputline ){
	int i = 0;
	while( inputline[i] && (inputline[i] != '#')){
		i++;
	}
	inputline[i]='\0';
}


void cutwhitespace( char *inputline ){
	int i = 0;
	int j = 0;
	while( inputline[i] ){
		if( (inputline[i]!=' ') && (inputline[i]!='\t') ){
			inputline[j] = inputline[i];
			j++;
		}
		i++;
	}
	inputline[j]='\0';
}


int readinput ( tdata *calcdata ) {
	FILE *fp;
	char inputline[256];
	char strbuffer[256];
	double numbuffer, db[9];
	char c;
	char *p;
	char *q;
	int i, vip, in;
	
	
	if( !(fp=fopen(calcdata->infname,"rt")) ){
		printf("Error opening file: %s \n Program terminated. \n",calcdata->infname);
		return 10;
	}
	vip = 0;
	while(!feof(fp)){
		i=0;
		c=fgetc(fp);
		while( (i<255) && (c!=EOF) && (c!='\n') ) {
			inputline[i]=c;
			i++;
			c=fgetc(fp);
		}
		if(i>=255){
			printf("Inputline buffer overflow! \n Program terminated. \n");
			return 11;				
		}
		inputline[i]='\0';

		cutcomm(inputline);
		cutwhitespace(inputline);
		if( inputline[0] ){
			if( sscanf(inputline,"xslibfile=%s", strbuffer) ){
				strcpy( calcdata->xslibname, strbuffer );
				vip++;
				continue;
			}
			if( sscanf(inputline,"mx=%lf", &numbuffer) ){
				calcdata->mx = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"my=%lf", &numbuffer) ){
				calcdata->my = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"ux=%lf", &numbuffer) ){
				calcdata->ux = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"uy=%lf", &numbuffer) ){
				calcdata->uy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dx=%lf", &numbuffer) ){
				calcdata->dx = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dy=%lf", &numbuffer) ){
				calcdata->dy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"thick=%lf", &numbuffer) ){
				calcdata->thick = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"thick2=%lf", &numbuffer) ){
				calcdata->thick2 = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dist1=%lf", &numbuffer) ){
				calcdata->dist1 = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dist2=%lf", &numbuffer) ){
				calcdata->dist2 = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixm=%lf", &numbuffer) ){
				calcdata->pixm = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixd=%lf", &numbuffer) ){
				calcdata->pixd = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixu=%lf", &numbuffer) ){
				calcdata->pixu = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"efelb=%lf", &numbuffer) ){
				calcdata->efelb = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"energy=%lf", &numbuffer) ){
				calcdata->energy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"rho=%lf", &numbuffer) ){
				calcdata->rho = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"M=%lf", &numbuffer) ){
				calcdata->M = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"Z=%lf", &numbuffer) ){
				calcdata->Z = numbuffer;
				vip++;
				continue;
			}
			if( strcmp(inputline,"scatzmode=on") == 0 ){
				calcdata->scatzmode = 1;
				continue;
			}
			if( strcmp(inputline,"onescatmode=on") == 0 ){
				calcdata->onescatmode = 1;
				continue;
			}
			if( strcmp(inputline,"twoscatmode=on") == 0 ){
				calcdata->twoscatmode = 1;
				continue;
			}
			if( strcmp(inputline,"abszmode=on") == 0 ){
				calcdata->abszmode = 1;
				continue;
			}
			if( strcmp(inputline,"doppler=on") == 0 ){
				calcdata->doppler = 1;
				continue;
			}
			if( sscanf(inputline,"dopplerdata=%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf",
						db,db+1,db+2,db+3,db+4,db+5,db+6,db+7,db+8) ){
				for( i=0; i<9; i++) calcdata->dopplerdata[i] = db[i];
				vip++;
				continue;
			}
			if( sscanf(inputline,"scatzdist=%lf", &numbuffer) ){
				calcdata->scatzdist = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"abszdist=%lf", &numbuffer) ){
				calcdata->abszdist = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"maxiter=%lf", &numbuffer) ){
				calcdata->maxiter = (int)numbuffer;
				continue;
			}
			if( sscanf(inputline,"printiter=%lf", &numbuffer) ){
				calcdata->printiter = (int)numbuffer;
				continue;
			}
			if( strcmp(inputline,"sourcemode=points") == 0 ){
				calcdata->sourcemode = POINTS;
				vip++;
				continue;
			}		
			if( strcmp(inputline,"sourcemode=line") == 0 ){
				calcdata->sourcemode = LINE;
				vip++;
				continue;
			}		
			if( strcmp(inputline,"sourcemode=circle") == 0 ){
				calcdata->sourcemode = RING;
				vip++;
				continue;
			}
			if( strcmp(inputline,"sourcemode=nti") == 0 ){
				calcdata->sourcemode = NTI;
				vip++;
				continue;
			}				
			if( sscanf(inputline,"activity=%lf", &numbuffer) ){
				calcdata->activity = numbuffer;
				vip++;
				continue;
			}
			if( strcmp(inputline,"energynoise=on") == 0 ){
				calcdata->energynoise = 1;
				continue;
			}
			if( strcmp(inputline,"posscatnoise=on") == 0 ){
				calcdata->posscatnoise = 1;
				continue;
			}
			if( strcmp(inputline,"background=on") == 0 ){
				calcdata->background = 1;
				continue;
			}
			if( strcmp(inputline,"posabsnoise=on") == 0 ){
				calcdata->posabsnoise = 1;
				continue;
			}
			if( sscanf(inputline,"energyfwhm=%lf", &numbuffer) ){
				calcdata->energyfwhm = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"posscatfwhm=%lf", &numbuffer) ){
				calcdata->posscatfwhm = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"backgract=%lf", &numbuffer) ){
				calcdata->backgract = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"posabsfwhm=%lf", &numbuffer) ){
				calcdata->posabsfwhm = numbuffer;
				vip++;
				continue;
			}
			if( strncmp(inputline,"sourcedata=",11) == 0 ){
				p = inputline + 11;
				q = p;
				calcdata->sourcedatanum = 0;
				in = (*q != '\0');
				while (in && calcdata->sourcedatanum < MAXSOURCEDATA) {
					while (*q != ',' && *q != '\0') q++;
					in = (*q != '\0');
					*q = '\0';
					sscanf(p,"%lf",&(calcdata->sourceloc[calcdata->sourcedatanum]));
					calcdata->sourcedatanum++;
					q++;
					p = q;
				}
				vip++;
				continue;
			}		
		}
	}
	fclose(fp);
	if( vip!=( 22 + calcdata->scatzmode + calcdata->abszmode + calcdata->doppler + 
			calcdata->energynoise + calcdata->posscatnoise + calcdata->posabsnoise + 
				calcdata->background ) ) {
		printf("vip: %d \n",vip);
		printf("Imperfect inputfile. \n Program terminated. \n");
		return 12;
	}
	if( (calcdata->sourcemode == LINE && calcdata->sourcedatanum != 7)
	 || (calcdata->sourcemode == RING && calcdata->sourcedatanum != 4)
	 || (calcdata->sourcemode == POINTS && calcdata->sourcedatanum %3 != 0 )) {
		printf("Wrong geometry data. \n Program terminated. \n");
		return 12;
	}
	
	return 0;
}



/*******  foton data processing routines  *************************************************************************************************/



int insertdigifoto ( long int index, int multiscatter, tdata *calcdata ) {
	tdigifoto *temp;
	tdigifoto *kukker;
	
	calcdata->dfpointer = calcdata->dflisthead;
	if ( calcdata->dfpointer ) {
		calcdata->dfpointer = calcdata->dflisthead;

		if( calcdata->dfpointer->index > index){
			if( !(calcdata->dflisthead = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
				printf("Memory allocation error. \n Program terminated. \n");
				return 201;
			}
			calcdata->digifotonum++;
			calcdata->dflisthead->index = index;
			calcdata->dflisthead->n = 1;
			calcdata->dflisthead->multiscatter = multiscatter;
			calcdata->dflisthead->next = calcdata->dfpointer;
		} else {
			while( (calcdata->dfpointer) && (calcdata->dfpointer->index<index) ){
				kukker = calcdata->dfpointer;
				calcdata->dfpointer = calcdata->dfpointer->next;
			}
			if(calcdata->dfpointer) {
				if(calcdata->dfpointer->index == index) {
					calcdata->dfpointer->n++;
				} else {
					if( !(temp = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
						printf("Memory allocation error. \n Program terminated. \n");
						return 202;
					}
					calcdata->digifotonum++;
					temp->index = index;
					temp->multiscatter = multiscatter;
					temp->n = 1;
					kukker->next = temp;
					temp->next = calcdata->dfpointer;
				}
			} else {
				if( !(temp = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
					printf("Memory allocation error. \n Program terminated. \n");
					return 203 ;
				}
				calcdata->digifotonum++;
				temp->index = index;
				temp->multiscatter = multiscatter;
				temp->n = 1;
				kukker->next = temp;
				temp->next = calcdata->dfpointer;				
			}
		}
	} else {
		if( !(calcdata->dflisthead = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 204;
		}
		calcdata->digifotonum++;
		calcdata->dflisthead->index = index;
		calcdata->dflisthead->multiscatter = multiscatter;
		calcdata->dflisthead->n = 1;
		calcdata->dflisthead->next = NULL;
	}
	return 0;
}


long int digit( double xd, double yd, double xu, double yu, double denergy, tdata *calcdata ){
	int maxxd, maxyd, maxxu, maxyu;
	int offsxd, offsyd, offsxu, offsyu;
	int e, u, d;
	int uxi, uyi, dxi, dyi;
	long int index;

	uxi=floor(xu/calcdata->pixu);
	uyi=floor(yu/calcdata->pixu);

	dxi=floor(xd/calcdata->pixd);
	dyi=floor(yd/calcdata->pixd);

	maxxd = round( calcdata->dx/calcdata->pixd );
	maxyd = round( calcdata->dy/calcdata->pixd );
	offsxd = round( maxxd / 2.0 );
	offsyd = round( maxyd / 2.0);
	d = round( (dxi+offsxd)+(dyi+offsyd)*maxxd );

	maxxu = round( calcdata->ux/calcdata->pixu );
	maxyu = round( calcdata->uy/calcdata->pixu );
	offsxu = round( maxxu / 2.0 );
	offsyu = round( maxyu / 2.0 );
	u = round( (uxi+offsxu)+(uyi+offsyu)*maxxu);

	e = floor( denergy/calcdata->energy * round(100.0/calcdata->efelb) );
	
	index =	e+ 
		u*round(100.0/calcdata->efelb)+
		d*round( maxxu*maxyu ) * round(100.0/calcdata->efelb);
	return index;
}


void filldfvect( tdata *calcdata ) {
	long int i;
	calcdata->dfpointer = calcdata->dflisthead;
	for( i=0; i<calcdata->digifotonum; i++ ){
		calcdata->dfvect[i].index = calcdata->dfpointer->index;
		calcdata->dfvect[i].multiscatter = calcdata->dfpointer->multiscatter;
		calcdata->dfvect[i].n = calcdata->dfpointer->n;
		calcdata->dfvect[i].nonzeroprob = 0;
		calcdata->dfpointer = calcdata->dfpointer->next;
	}
}


/*******  simulation  *********************************************************************************************************************/


int readxslib( tdata *calcdata ) {

///	calcdata->xsec[5][142]
//
	FILE * fp;
	int i;
	int j,k,in;
	char c;
	char *p, *q;
	char inputline[4000];

	fp = fopen(calcdata->xslibname, "rt");
	if( fp==NULL ) {
		printf("Error opening file: %s \n",calcdata->xslibname);
		return 29;
	}

	j = 0;
	
	while(!feof(fp) && j<5){
		i=0;
		c=fgetc(fp);
		while( (i<4000) && (c!=EOF) && (c!='\n') ) {
			inputline[i]=c;
			i++;
			c=fgetc(fp);
		}
		if(i>=4000){
			printf("Inputline buffer overflow! \n Program terminated. \n");
			return 16;				
		}
		inputline[i]='\0';
		cutcomm(inputline);
		cutwhitespace(inputline);
		if(inputline[0] != '\0'){
		/*	printf("%s \n",inputline);*/
			p = inputline;
			q = p;
			in = (*q != '\0');
			k = 0;	
			while (in && k<142) {
				while (*q != ',' && *q != '\0') q++;
				in = (*q != '\0');
				*q = '\0';
				sscanf(p,"%lf",&(calcdata->xsec[j][k]));
			/*	printf("%d, %d ==> %lf\n",j,k,calcdata->xsec[j][k]);*/
				k++;
				q++;
				p = q;
			}
			j++;
		}
	}
	fclose(fp);
	return 0;
}




double rnd_num( double x ){
	return x*rand()/(double)RAND_MAX;
}

void background (tdata *calcdata, double *sourcepoint){

	sourcepoint[0]=rnd_num(calcdata->mx)-calcdata->mx/2;
	sourcepoint[1]=rnd_num(calcdata->my)-calcdata->my/2;
	sourcepoint[2]=calcdata->sourceloc[2];
	if (calcdata->sourcemode==LINE) sourcepoint[2]=calcdata->sourceloc[6];
}


void sourcelocation ( tdata *calcdata, double *sourcepoint ) {
	int actpoint, a;
	double r1, pointx, pointy;
	
	if(calcdata->sourcemode==POINTS){
		actpoint=floor(rnd_num((double)calcdata->numpoints));
		a=3*actpoint;
		sourcepoint[0]=calcdata->sourceloc[a];
		sourcepoint[1]=calcdata->sourceloc[a+1];
		sourcepoint[2]=calcdata->sourceloc[a+2];
	}
	if(calcdata->sourcemode==LINE){
		if(calcdata->sourceloc[0]==calcdata->sourceloc[1]){
			sourcepoint[1]=rnd_num(calcdata->sourceloc[5]-calcdata->sourceloc[4])+calcdata->sourceloc[4];
			sourcepoint[0]=calcdata->sourceloc[0];
		}
		else{
			sourcepoint[0]=rnd_num(calcdata->sourceloc[1]-calcdata->sourceloc[0])+calcdata->sourceloc[0];
			sourcepoint[1]=calcdata->sourceloc[2]*sourcepoint[0]+calcdata->sourceloc[3];
		}
		sourcepoint[2]=calcdata->sourceloc[6];
	}
	if(calcdata->sourcemode==RING){
		do{
			pointx=rnd_num(calcdata->sourceloc[3]);
			pointy=rnd_num(calcdata->sourceloc[3]);
			r1=sqrt(pointx*pointx+pointy*pointy);
		}while(r1>calcdata->sourceloc[3]);
		sourcepoint[0]=pointx/r1+calcdata->sourceloc[0];
		sourcepoint[1]=pointy/r1+calcdata->sourceloc[1];
		sourcepoint[2]=calcdata->sourceloc[2];
	}
	if(calcdata->sourcemode==NTI){
		actpoint=floor(rnd_num((double)6));
		if(actpoint==0){
			sourcepoint[0]=-2.0;
			sourcepoint[1]=rnd_num((double)3)-1.5;
		}
		if(actpoint==1){
			sourcepoint[1]=rnd_num((double)3)-1.5;
			sourcepoint[0]=-(2.5+sourcepoint[1])/2;
		}
		if(actpoint==2){
			sourcepoint[0]=-0.5;
			sourcepoint[1]=rnd_num((double)3)-1.5;
		}
		if(actpoint==3){
			sourcepoint[0]=rnd_num(1.5);
			sourcepoint[1]=2;
		}
		if(actpoint==4){
			sourcepoint[0]=0.75;
			sourcepoint[1]=rnd_num((double)3)-1.5;
		}
		if(actpoint==5){
			sourcepoint[0]=2.0;
			sourcepoint[1]=rnd_num((double)3)-1.5;
		}
		sourcepoint[2]=calcdata->sourceloc[2];
	}
/*	printf("\nsource: %lf, %lf, %lf", sourcepoint[0], sourcepoint[1], sourcepoint[2]); */ 

}


void isotrop( double *actdir ) {
	int i;
	double r2;
	
	do{
		for(i=0;i<3;i++){
			actdir[i]=rnd_num(2)-1;
		}
		r2=sqrt(actdir[0]*actdir[0]+actdir[1]*actdir[1]+actdir[2]*actdir[2]);
	}while(r2>1);
	actdir[0]=actdir[0]/r2;
	actdir[1]=actdir[1]/r2;
	actdir[2]=actdir[2]/r2;
/*	printf("\nsourcedir: %lf, %lf, %lf", actdir[0], actdir[1], actdir[2]); */

}

double kahn (double actenergy){

	double r3;
	int i3;	
	
	do{
		if(rnd_num(1)<((1+2/(0.511/actenergy))/(9+2/(0.511/actenergy)))){
			r3=1+rnd_num(2)/(0.511/actenergy);
			if(rnd_num(1)<=4*(1/r3-1/(r3*r3))) i3=1;
			else i3=0;
		}
		else{
			r3=(1+2/(0.511/actenergy))/(1+rnd_num(2)/(0.511/actenergy));
			if(rnd_num(1)<=0.5*(pow((0.511/actenergy)-r3*(0.511/actenergy)+1,2)+1/r3)) i3=1;
			else i3=0;
		}
	}while(i3==0);
	return actenergy/r3;
}

double gauss (double fwhm){

	int i;
	double g;

	g=-6;
	for(i=0; i<12; i++) g+=rnd_num(1);
	return g*fwhm;
}

	
int onephotonplay(tdata *calcdata, double *rndd, long int i){

	int j, n, numscat, out, helpdist;
	double scatpos[30], scatangle, dopplerfwhm;
	double sourcepoint[3], scatlevelz, abslevelz;
	double act[3], actdir[3], actenergy, befscatenergy, rnddist;
	double calcdir[3], uplusv, beta, r4, abspos[3];
	int errorcode;

	errorcode = 0;

	if ( calcdata->background==1 && i<calcdata->backgract) background( calcdata, sourcepoint);
	else sourcelocation( calcdata, sourcepoint );
	
	isotrop( actdir );
	
	if(actdir[2]>0){
		
		act[0]=sourcepoint[0]+(-actdir[0]*(sourcepoint[2])/actdir[2]);
		act[1]=sourcepoint[1]+(-actdir[1]*(sourcepoint[2])/actdir[2]);
		act[2]=floor(sourcepoint[2]+(-actdir[2]*(sourcepoint[2])/actdir[2]));
/*	printf("\n%lf, %lf, %lf", act[0], act[1], act[2]);  */
		/*foton az alsó detektor aljánál*/
		
		if(	act[0] < calcdata->scatdetxmax && act[0] > calcdata->scatdetxmin &&
			act[1] < calcdata->scatdetymax && act[1] > calcdata->scatdetymin){
			/*ha ez el is találja az alsó detektort*/
/*		printf("\nbent");   */
			actenergy=calcdata->energy;
			befscatenergy=calcdata->energy;
			numscat=0;
			out=0;
			while(act[2] >= 0 && act[2] < calcdata->thick && 
			      act[0] < calcdata->scatdetxmax && act[0] > calcdata->scatdetxmin &&
			      act[1] < calcdata->scatdetymax && act[1] > calcdata->scatdetymin && 
			      out==0){
			
/*			printf("\nszorasfuggveny");  */
				n=0;
				while(calcdata->xsec[0][n]<actenergy ) n++;
/*			printf("\nE=%d", n); */
				rnddist=-log(rnd_num(1))/(calcdata->xsec[calcdata->scatdetmat+1][n]*calcdata->xsec[calcdata->scatdetmat][0]);
				helpdist=floor(rnddist*10);
				if(helpdist>50) rndd[50]++;
				else rndd[helpdist]++;

				scatpos[0]=act[0];
				scatpos[1]=act[1];
				scatpos[2]=act[2];
				if(calcdata->posscatnoise==1){
					for(i=0; i<3; i++) scatpos[i]+=gauss(calcdata->posscatfwhm);
				}
				act[0]=act[0]+actdir[0]*rnddist;
		        	act[1]=act[1]+actdir[1]*rnddist;
				act[2]=act[2]+actdir[2]*rnddist;
/*			printf("\n %lf", rnddist); */

				if(calcdata->scatzmode==1){
					act[0]=act[0]+(actdir[0]*scatlevelz/actdir[2]);
					act[1]=act[1]+(actdir[1]*scatlevelz/actdir[2]);
					act[2]=act[2]+(actdir[2]*scatlevelz/actdir[2]);
				}
				
				if(act[2]>=0 && act[2] < calcdata->thick && 
				   act[0] < calcdata->scatdetxmax && act[0] > calcdata->scatdetxmin &&
				   act[1] < calcdata->scatdetymax && act[1] > calcdata->scatdetymin) {
					if( rnd_num(1) < (calcdata->xsec[calcdata->scatdetmat][n]/calcdata->xsec[calcdata->scatdetmat+1][n])){
						numscat++;
/*					printf("\n szór"); */
						calcdata->ejj++;
						actenergy=kahn(actenergy);
						if(calcdata->energynoise==1) actenergy+=gauss(calcdata->energyfwhm);
						scatangle=acos(1+0.511*(1/befscatenergy-1/actenergy));
						befscatenergy=actenergy;
						if(calcdata->doppler==1){
							if(calcdata->dopplerdata[0]<scatangle) dopplerfwhm=calcdata->dopplerdata[1]*
								pow(scatangle,3)+calcdata->dopplerdata[2]*pow(scatangle,2)+
								calcdata->dopplerdata[3]*scatangle+calcdata->dopplerdata[4];
							else dopplerfwhm=calcdata->dopplerdata[5]*pow(scatangle,3)+
								calcdata->dopplerdata[6]*pow(scatangle,2)+
								calcdata->dopplerdata[7]*scatangle+calcdata->dopplerdata[8];
							scatangle+=gauss(dopplerfwhm);
						}
						beta=rnd_num(TWOPI);
						uplusv=sqrt(pow(actdir[0],2)+pow(actdir[1],2));
						calcdir[0]=(actdir[1]/uplusv)*(sin(scatangle)*cos(beta))+
							(actdir[0]*actdir[2]/uplusv)*(sin(scatangle)*sin(beta))+actdir[0]*cos(scatangle);
						calcdir[1]=(-actdir[0]/uplusv)*(sin(scatangle)*cos(beta))+
							(actdir[1]*actdir[2]/uplusv)*(sin(scatangle)*sin(beta))+actdir[1]*cos(scatangle);
						calcdir[2]=(-uplusv*sin(scatangle)*sin(beta))+actdir[2]*cos(scatangle);
						r4=sqrt(calcdir[0]*calcdir[0]+calcdir[1]*calcdir[1]+calcdir[2]*calcdir[2]);
/*					printf("%lf", r4);*/
						actdir[0]=calcdir[0]/r4;
						actdir[1]=calcdir[1]/r4;
						actdir[2]=calcdir[2]/r4;
/*					printf("\n %lf %lf %lf", actdir[0], actdir[1], actdir[2]);*/
					}
					else return 0;
				}
				if(calcdata->onescatmode==1){
					if(numscat==1){
						out=1;
						scatpos[0]=act[0];
						scatpos[1]=act[1];
						scatpos[2]=act[2];
						if(calcdata->posscatnoise==1){
		                                         for(i=0; i<3; i++) scatpos[i]+=gauss(calcdata->posscatfwhm);
						}
						 
					}
				}
				if(calcdata->twoscatmode==1){
					if(numscat==2){
						out=1;
						scatpos[0]=act[0];
						scatpos[1]=act[1];
						scatpos[2]=act[2];
						if(calcdata->posscatnoise==1){
							for(i=0; i<3; i++) scatpos[i]+=gauss(calcdata->posscatfwhm);
		                                }
						 
					}
				}
			
			} /* a while ciklus vége */

			
			if(calcdata->twoscatmode==numscat) return errorcode;
			if(numscat!=0){
				
				if( actdir[2]>0 ){
					act[0]=act[0]+(actdir[0]*(calcdata->thick+calcdata->dist2-act[2])/actdir[2]);
					act[1]=act[1]+(actdir[1]*(calcdata->thick+calcdata->dist2-act[2])/actdir[2]);
					act[2]=act[2]+(actdir[2]*(calcdata->thick+calcdata->dist2-act[2])/actdir[2]);
					if(calcdata->absdetxmin<act[0] && act[0]<calcdata->absdetxmax &&
							calcdata->absdetymin<act[1] && act[1]<calcdata->absdetymax){
/*						printf("\n%lf %lf %lf", act[0], act[1], act[2]);*/
						n=0;
						while(calcdata->xsec[0][n]<actenergy ){
							n++;
						}
						rnddist=-1/calcdata->xsec[calcdata->absdetmat+1][n]*calcdata->xsec[calcdata->absdetmat][0]*log(rnd_num(1));
	/*					printf("\n%d %lf", n, rnddist); */
						act[0]=act[0]+(actdir[0]*rnddist);
						act[1]=act[1]+(actdir[1]*rnddist);
						act[2]=act[2]+(actdir[2]*rnddist);
	/*					printf("\n%lf %lf %lf %lf", act[0], act[1], act[2], calcdata->thick+calcdata->dist2+calcdata->thick2); */ 
						if( act[2] < calcdata->thick + calcdata->dist2 + calcdata->thick2 && 
						    act[0] < calcdata->absdetxmax &&
						    act[0] > calcdata->absdetxmin && act[1] < calcdata->absdetymax && 
						    act[1] > calcdata->absdetymin 
						    /* && (rnd_num(1)<(calcdata->xsec[calcdata->absdetmat][n]/
						     * calcdata->xsec[calcdata->absdetmat+1][n]))*/){
/*							printf("\nbasszameg");*/	
							for(j=0; j<3; j++){
								abspos[j]=act[j];
							}
						
							if(calcdata->abszmode==1){
								
								abspos[0]=act[0]+(actdir[0]*abslevelz/actdir[2]);
								abspos[1]=act[1]+(actdir[1]*abslevelz/actdir[2]);
								abspos[2]=act[2]+(actdir[2]*abslevelz/actdir[2]);
							}
							if(calcdata->posabsnoise==1){
								for(j=0; j<3; j++) abspos[j]+=gauss(calcdata->posabsfwhm);
			                                }
							 
	
			/*			printf("\t%lf\t%lf\t%lf\t%lf\t%d\t%lf\t%lf\t%lf\n",	
							scatpos[0], scatpos[1], scatpos[2], 
							(calcdata->energy-actenergy), numscat,
							abspos[0:], abspos[1], abspos[2]);*/
	
							if( errorcode = insertdigifoto( 
							    digit(scatpos[0], scatpos[1], abspos[0], abspos[1], 
							    (calcdata->energy-actenergy), calcdata), (1<numscat), calcdata) ) {
								printf("Insert failed. \n");
								return errorcode;
							}					
						}
					}
				}
			}	
		}
	}	
	return 0;
}



/*******  print result  *******************************************************************************************************************/


int yeah ( int n, tdata *calcdata ) {
	char outfn[256];
	FILE *fp;
	long int l;
	int x,y, gx,gy;

	printf("After %d. iteration: \n",n);
	sprintf(outfn,"%s.output%d", calcdata->infname, n);
	printf("\tWriting results to: %s ... \n",outfn);
	if( !(fp=fopen(outfn,"wt")) ){
		printf("Error opening file: %s. \n Program terminated. \n",outfn);
		return 40;
	}
	
	gx = round(calcdata->mx / calcdata->pixm);
	gy = round(calcdata->my / calcdata->pixm);
	l = 0;
	for( y=0; y<gy; y++ ){
		for( x=0; x<gx; x++ ){
			fprintf(fp,"%.5E\t",calcdata->lambda[calcdata->flipflop][l]);
			l++;
		}
		fprintf(fp,"\n");
	}
	fflush(fp);
	fclose(fp);
	printf("done\n");
	return 0;
}


/*******  calculating probmx  *************************************************************************************************************/


double kleinnishina ( tdata *calcdata, double costheta, double outenergy, double rdu) {
	double gamma, amounte, rnull,  sintheta, dtheta, prob21, prob22;

	amounte=calcdata->rho/calcdata->M*AVOGADRO*calcdata->Z*(calcdata->pixd*calcdata->pixd*calcdata->thick);
	gamma=calcdata->energy/0.511;
	rnull=2.818*10E-13;
	sintheta=sqrt(1-costheta*costheta);
	dtheta=calcdata->pixu*(calcdata->dist2+calcdata->thick/2.0)/pow(rdu,2);
	prob21=2*PI*pow(rnull,2)*sintheta*dtheta*((1+pow(costheta,2))/2.0)*(1.0/pow((1+gamma*(1-costheta)),2));
	prob22=( pow(gamma,2)*pow( (1-costheta),2 ) ) / ( (1+pow(costheta,2)) * (1+gamma*(1-costheta)) );
	return((prob21*(1+prob22))*amounte);
}


void returnofjimi ( tdata *calcdata ) {	/* in memoriam Jimi Hendrix */
	double xu, yu, zu;
	double xd, yd, zd;
	double xm, ym, zm;
	double rmd, smd, rdu;
	double v1x, v1y, v1z;
	double v2x, v2y, v2z;
	//double pux, puy, puz;
	double alpha, costheta, sintheta;
	double outenergy, scatenergy;
	double reff;
	
	double prob, prob1, prob2, prob3;
	
	long int col,row;
	long int index;
	
	int numiu, numju, maxiu, maxju;
	int numid, numjd, maxid, maxjd;
	int numim, numjm, maxim, maxjm;

	double kicsi, nagy;
	
	kicsi = 0.2;
	nagy = 1 - kicsi;
	
	maxiu = round(calcdata->ux / calcdata->pixu);
	maxju = round(calcdata->uy / calcdata->pixu);
	maxid = round(calcdata->dx / calcdata->pixd);
	maxjd = round(calcdata->dy / calcdata->pixd);
	maxim = round(calcdata->mx / calcdata->pixm);
	maxjm = round(calcdata->my / calcdata->pixm);
	zu=calcdata->dist2+calcdata->thick;
	
	reff =  calcdata->pixu / sqrt(PI);

	for( numjm = 0; numjm<maxjm; numjm++ ) {
	for( numim = 0; numim<maxim; numim++ ) {

		col=round(numim+numjm*(calcdata->mx/calcdata->pixm));
		row = 0;
		calcdata->colsum[col] = 0;
						
			
		xm=-(calcdata->mx/2.0)+(numim*calcdata->pixm+(calcdata->pixm/2.0));
		ym=-(calcdata->my/2.0)+(numjm*calcdata->pixm+(calcdata->pixm/2.0));
		zm=calcdata->dist1*(-1);

		for( numjd = 0; numjd<maxjd; numjd++ ) {
		for( numid = 0; numid<maxid; numid++ ) {

			xd=-(calcdata->dx/2.0)+(numid*calcdata->pixd+(calcdata->pixd/2.0));
			yd=-(calcdata->dy/2.0)+(numjd*calcdata->pixd+(calcdata->pixd/2.0));
			zd=calcdata->thick/2.0;
			rmd=sqrt(pow((xm-xd),2)+pow((ym-yd),2)+pow(zm-zd,2));
			smd=sqrt(pow((xm-xd),2)+pow((ym-yd),2));
			alpha=asin(smd/rmd);
			prob1=((pow(calcdata->pixd,2)/cos(alpha))/(pow(rmd,2)*4*PI));

			v1x=(xd-xm)/rmd;
			v1y=(yd-ym)/rmd;
			v1z=(zd-zm)/rmd;

			for( numju = 0; numju<maxju; numju++ ) {
			for( numiu = 0; numiu<maxiu; numiu++ ) {
	
							
							
				xu=-(calcdata->ux/2.0)+(numiu*calcdata->pixu+(calcdata->pixu/2.0));
				yu=-(calcdata->uy/2.0)+(numju*calcdata->pixu+(calcdata->pixu/2.0));
				rdu=sqrt(pow(xu-xd,2)+pow(yu-yd,2)+pow(zu-zd,2));
				v2x=(xu-xd)/rdu;
				v2y=(yu-yd)/rdu;
				v2z=(zu-zd)/rdu;
				costheta=v1x*v2x+v1y*v2y+v1z*v2z;
				sintheta = sqrt(1-costheta*costheta);
				outenergy=1.0/((1.0/calcdata->energy)+(1-costheta)/0.511);
				scatenergy=calcdata->energy-outenergy;
				prob2=kleinnishina(calcdata, costheta, outenergy, rdu);
				if( (rdu*PI*sintheta)/reff>1 ) prob3=reff/(rdu*PI*sintheta);
				else prob3 = 1;

				index =	floor(scatenergy/calcdata->energy * 100.0/calcdata->efelb) + 
					round(100.0/calcdata->efelb)* (numju*maxiu+numiu) +
					round(100.0/calcdata->efelb)* maxiu*maxju * (numjd*maxid+numid);
				prob = prob1*prob2*prob3;
				calcdata->colsum[col] += prob;
				if( row < calcdata->digifotonum && prob ) {
					if( calcdata->doppler==1 || calcdata->energynoise==1 ) {
						while( row < calcdata->digifotonum && calcdata->dfvect[row].index < index -1 ) row++;		
						if( row < calcdata->digifotonum && calcdata->dfvect[row].index == index-1 ){
							calcdata->probmx[row][col] += prob* double(kicsi) / 2.0;
							calcdata->dfvect[row].nonzeroprob = 1;
							row++;
						}
						if( row < calcdata->digifotonum && calcdata->dfvect[row].index == index){
							calcdata->probmx[row][col] += prob* nagy;
							calcdata->dfvect[row].nonzeroprob = 1;
							row++;
						}
						if( row < calcdata->digifotonum && calcdata->dfvect[row].index == index+1){
							calcdata->probmx[row][col] += prob* double(kicsi) / 2.0;
							calcdata->dfvect[row].nonzeroprob = 1;
						}
							

					} else {
						while( row < calcdata->digifotonum && calcdata->dfvect[row].index < index) row++;
						if( row < calcdata->digifotonum && 
						    calcdata->dfvect[row].index == index ) {
							calcdata->probmx[row][col] = prob;
							calcdata->dfvect[row].nonzeroprob = 1;
							row++;
						}
					}
				}
			}
			}			
		}
		}			
	}
	}
}


/*******  let's rock!  ********************************************************************************************************************/


void imagegeneration ( tdata *calcdata ) {
	int n;
	long int j,i,k;
	int flip,flop;
	double basszus, violin;
	
	for( n=1; n<=calcdata->maxiter; n++){
		flip = (n+1) % 2;
		flop = n % 2;
			
		for( j=0; j<calcdata->pmxcolnum; j++ ){
			basszus = 0;

			for( i=0; i<calcdata->digifotonum; i++) {
				if(calcdata->dfvect[i].nonzeroprob){
					violin = 0;
					for( k=0; k<calcdata->pmxcolnum; k++ ) {
						violin += calcdata->probmx[i][k] * calcdata->lambda[flop][k];
					}
					if (violin) basszus += calcdata->probmx[i][j] * calcdata->dfvect[i].n / violin;
				}
			}			
			calcdata->lambda[flip][j] = calcdata->lambda[flop][j] * (1.0/calcdata->colsum[j]) * basszus;
		}
		calcdata->flipflop = flip;
		if( n%calcdata->printiter == 0 ) yeah( n, calcdata );				
	}		
}


/*******  main  ***************************************************************************************************************************/


int main( int argc, char *argv[] ) {
	int errorcode;
	tdata calcdata; 						/* static structure, contains all data */
	long int jajj,jujj,jajjjujj,i;
	double rndd[51];
	
	printf("\n\n============================================================\n");
	printf("\n\n\t  Compton Camera image generator software.\n");	/* Welcome message */
	printf("\n\n\t\t version:\n\t\t\t 0.20 beta\n");
	printf("\t\t date:\n\t\t\t 8 November 2006\n");
	printf("\t\t author:\n\t\t\t Karoly Nagy\n\t\t\t Szilard Szalay\n");
	printf("\n\n============================================================\n");
	printf("\n\n");

	init ( &calcdata );						/* initializing datastructure I. - pointers */
	for(i=0; i<51; i++) rndd[i]=0;
	
	if(argc == 2){							/* need only one argument */
		strcpy(calcdata.infname, argv[1]);			/* which is the name of the inputile */
	} else {
		printf("Usage: %s <inputfile> \n", argv[0]);
		return 1;
	}

	printf("\nReading inputfile ... \n");
	if( errorcode = readinput ( &calcdata ) ) return errorcode;	/* reads the inputfile */
	printf("done. \n");

	setgeomdata( &calcdata );

	if( errorcode = readxslib ( &calcdata ) ) return errorcode;

	
	printf("\nSimulating: ... \n");
	for( i=0; i<calcdata.activity+calcdata.backgract; i++ ) {
/*		printf("\n%ld", i); */
		if( errorcode = onephotonplay ( &calcdata, rndd, i) ) return errorcode;
	}	
	
/*	printf("Dist:");
	for(i=0; i<51; i++) printf("\n%lf", rndd[i]); */
	
	printf("done. \n");
	printf("Number of scatters:%ld\n", calcdata.ejj);
	printf("Number of elements: %ld\n",calcdata.digifotonum);
/*	printf("Efficiency: %lf\n", (double)(calcdata.digifotonum/calcdata.activity));   */

	
	printf("\nInitializing dfvect ... \n");
	if( errorcode = initdfvect ( &calcdata ) ) return errorcode;	/* initializing datastructure II. - fotovectorn */
	printf("done. \n");

	printf("\nWriting fotons to dfvect ... \n");
	filldfvect ( &calcdata );					/* writing fotons to dfvect */
	printf("done. \n");

	freedflist( &calcdata ); 

	printf("\nInitializing pmatrix ... \n");
	if( errorcode = initpm ( &calcdata ) ) return errorcode;	/* initializing datastructure III. - probability matrix */
	printf("done. \n");						/* it needs the geometry and foton data */
	printf("Dimensions: %ld x %ld \n", calcdata.pmxcolnum, calcdata.pmxrownum);
	
	printf("\nLoading pmatrix ... \n");
	returnofjimi ( & calcdata );	/* calculates the probability matrix elements */
	printf("done. \n");
	printf("number of elements = %ld x %ld \n", calcdata.pmxcolnum, (long int)round(calcdata.pmxrownum*calcdata.efelb/100));



	jajj = 0;
	jujj = 0;
	jajjjujj = 0;
	for( i=0; i<calcdata.digifotonum; i++ ) {
		if(!calcdata.dfvect[i].nonzeroprob) jajj++;
		if(calcdata.dfvect[i].multiscatter) jujj++;
		if(!calcdata.dfvect[i].nonzeroprob && calcdata.dfvect[i].multiscatter) jajjjujj++;
	}
	printf("0valsegu                 foton : %ld\n",jajj);
	printf("Multiscatter             foton : %ld\n",jujj);
	printf("0valsegu && Multiscatter foton : %ld\n",jajjjujj);
	


	printf("\nInitializing lambda-list ... \n");
	if( errorcode = initlambda ( &calcdata ) ) return errorcode;	/* initializing datastructure IV. - lambda vector */
	printf("done. \n");
			
	printf("\nComputing started.\n");
	imagegeneration ( & calcdata );					/* iterative image generating algorythm */
	printf("computing done.\n\n");
	
	return 0;
}


/*
   
(. Y .)

*/

