/*******************************************************************************************************************************************


		Compton Camera image generator software

			version:
				0.18 beta
			date:
				8 November 2006
			author:
				Karoly Nagy
				Szilard Szalay


*******************************************************************************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define PI 3.1415
#define MAXEFELB 100
#define AVOGADRO 6.022E+23
#define AXIAL 1
#define	POLAR 2
#define DESCARTES 0


/*******  type definitions  ***************************************************************************************************************/


typedef struct digifoto {						/* this structure contains the data of fotons (element of a list) */
	long int index;							/*     have come on this specific path */
	long int n;							/*     n pieces */
	int multiscatter;
	struct digifoto *next;						/* pointer to the next element of this list */
} tdigifoto;


typedef struct {							/* the same as above, but this is an element of a vector */
	long int index;
	long int n;
	int multiscatter;
	int nonzeroprob;						/* TRUE, if "returnofjimi" give nonzero probability for this path */
} tdfvect;


typedef struct data {							/* All important data in this structure */
	char infname[255];						/* name of the inputfile */
	char ptracfname[255];						/* name of the MCNP-outputfile */
	int ptraccellfilter;
	
	tdigifoto *dflisthead;						/* headpointer of digitalized foton data list */
	tdigifoto *dfpointer;						/* pinter on digitalized foton data list */
	long int digifotonum;						/* number of d.f. */

	tdfvect *dfvect;
	
	double mx, my, dx, dy, dist1, ux, uy, dist2;			/* geometrical data */
	double pixm, pixd, pixu, thick;
	double uz, ur, pixphi;
	double efelb;							/* energy data */
	double energy;
	double rho, M, Z;						/* matter of scatter detector */

	int geometrymode;
	
	double *colsum;							/* pointer forcolsum-vector */
	double **probmx;						/* pointre for probability matrix*/
	long int pmxrownum, pmxcolnum;					/* numbers of rows and columns of pmatrix */

	double *lambda[2];						/* pointer for the data of steps of iteration */
	int flipflop;							/* index says whitch lambda[i] does contain 
									   the result of last iteration*/
	int maxiter;							/* number of iteration */
	int printiter;
} tdata;


typedef struct {
	int firstline[7];
	double secondline[9];
} tinputrecord;



/*******  initialization routines  ********************************************************************************************************/


void init ( tdata *calcdata ) {	
	calcdata->infname[0]='\0';
	calcdata->ptracfname[0]='\0';
	calcdata->ptraccellfilter = 0;

	calcdata->dflisthead = NULL;
	calcdata->dfpointer = NULL;
	calcdata->digifotonum = 0;

	calcdata->dfvect = NULL;

	calcdata->probmx = NULL;
	calcdata->colsum = NULL;
	calcdata->lambda[0] = NULL;
	calcdata->lambda[1] = NULL;
	calcdata->flipflop = 0;

	calcdata->maxiter = 100;
	calcdata->printiter = 5;
}


int initdfvect( tdata *calcdata ) {
	if( !(calcdata->dfvect = (tdfvect*)malloc(calcdata->digifotonum*sizeof(tdfvect)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 101;
	}
	return 0;
}


int initpm ( tdata *calcdata ) {
	long int row,col;

	calcdata->pmxcolnum = round(calcdata->mx * calcdata->my / pow(calcdata->pixm, 2));
	if(calcdata->geometrymode == DESCARTES ) {	
		calcdata->pmxrownum = round( calcdata->ux * calcdata->uy / pow(calcdata->pixu, 2) * 
						 calcdata->dx * calcdata->dy / pow(calcdata->pixd, 2) *
						 100.0/calcdata->efelb);
	} else {
		if(calcdata->geometrymode == AXIAL ) {
			calcdata->pmxrownum = round( round(2.0*PI/calcdata->pixphi) * 
							 calcdata->uz / calcdata->pixu  * 
							 calcdata->dx * calcdata->dy / pow(calcdata->pixd, 2) *
							 100.0/calcdata->efelb);
		}
	}
	
	if( !(calcdata->colsum = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 121 ;
	}

	if( !(calcdata->probmx = (double**)malloc(calcdata->digifotonum*sizeof(double*)) )) {
		printf("Memory allocation error. \n Program terminated. \n");
		return 122;
	}

	for( row = 0; row<calcdata->digifotonum; row++ ) {
		if( !(calcdata->probmx[row] = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 123;
		}
	}
	
	for( row = 0; row < calcdata->digifotonum; row++ ) {
	        for( col = 0; col < calcdata->pmxcolnum; col++ ) {
			calcdata->probmx[row][col] = 0;
		}
	}
	
	return 0;
}


int initlambda ( tdata * calcdata ) {
	long int l;
	int li;

	for( li=0; li<2; li++ ) {
		if( !(calcdata->lambda[li] = (double*)malloc(calcdata->pmxcolnum*sizeof(double)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 131;
		}
		for( l=0; l<calcdata->pmxcolnum; l++ ) {
			calcdata->lambda[li][l] = 1;
		}
	}
	return 0;
}


/*******  freeing unused memory  **********************************************************************************************************/


void freedflist( tdata *calcdata ) {
	tdigifoto *kukker;
	
	calcdata->dfpointer = calcdata->dflisthead;
	calcdata->dflisthead = NULL;
	while( calcdata->dfpointer ) {
		kukker = calcdata->dfpointer;
		calcdata->dfpointer = calcdata->dfpointer->next;
		free(kukker);
	}
}


/*******  reading inputfile  **************************************************************************************************************/


void cutcomm( char *inputline ){
	int i = 0;
	while( inputline[i] && (inputline[i] != '#')){
		i++;
	}
	inputline[i]='\0';
}


void cutwhitespace( char *inputline ){
	int i = 0;
	int j = 0;
	while( inputline[i] ){
		if( (inputline[i]!=' ') && (inputline[i]!='\t') ){
			inputline[j] = inputline[i];
			j++;
		}
		i++;
	}
	inputline[j]='\0';
}


int readinput ( tdata *calcdata ) {
	FILE *fp;
	char inputline[256];
	char strbuffer[256];
	double numbuffer;
	char c;
	int i, vip;
	
	
	if( !(fp=fopen(calcdata->infname,"rt")) ){
		printf("Error opening file: %s \n Program terminated. \n",calcdata->infname);
		return 10;
	}
	vip = 0;
	while(!feof(fp)){
		i=0;
		c=fgetc(fp);
		while( (i<255) && (c!=EOF) && (c!='\n') ) {
			inputline[i]=c;
			i++;
			c=fgetc(fp);
		}
		if(i>=255){
			printf("Inputline buffer overflow! \n Program terminated. \n");
			return 11;				
		}
		inputline[i]='\0';

		cutcomm(inputline);
		cutwhitespace(inputline);
		if( inputline[0] ){
			if( sscanf(inputline,"ptracfile=%s", strbuffer) ){
				strcpy( calcdata->ptracfname, strbuffer );
				vip++;
				continue;
			}
			if( strcmp(inputline,"geometry=descartes") == 0 ){
				calcdata->geometrymode = DESCARTES;
				vip++;
				continue;
			}
			if( strcmp(inputline,"geometry=axial") == 0 ){
				calcdata->geometrymode = AXIAL;
				vip++;
				continue;
			}
			if( strcmp(inputline,"geometry=polar") == 0 ){
				calcdata->geometrymode = POLAR;
				vip++;
				continue;
			}
			if( sscanf(inputline,"cellfilter=%lf", &numbuffer) ){
				calcdata->ptraccellfilter = (int)numbuffer;
				continue;
			}
			if( sscanf(inputline,"mx=%lf", &numbuffer) ){
				calcdata->mx = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"my=%lf", &numbuffer) ){
				calcdata->my = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"ux=%lf", &numbuffer) ){
				calcdata->ux = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"uy=%lf", &numbuffer) ){
				calcdata->uy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"uz=%lf", &numbuffer) ){
				calcdata->uz = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"ur=%lf", &numbuffer) ){
				calcdata->ur = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dx=%lf", &numbuffer) ){
				calcdata->dx = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dy=%lf", &numbuffer) ){
				calcdata->dy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"thick=%lf", &numbuffer) ){
				calcdata->thick = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dist1=%lf", &numbuffer) ){
				calcdata->dist1 = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"dist2=%lf", &numbuffer) ){
				calcdata->dist2 = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixm=%lf", &numbuffer) ){
				calcdata->pixm = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixd=%lf", &numbuffer) ){
				calcdata->pixd = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixphi=%lf", &numbuffer) ){
				calcdata->pixphi = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"pixu=%lf", &numbuffer) ){
				calcdata->pixu = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"efelb=%lf", &numbuffer) ){
				calcdata->efelb = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"energy=%lf", &numbuffer) ){
				calcdata->energy = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"rho=%lf", &numbuffer) ){
				calcdata->rho = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"M=%lf", &numbuffer) ){
				calcdata->M = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"Z=%lf", &numbuffer) ){
				calcdata->Z = numbuffer;
				vip++;
				continue;
			}
			if( sscanf(inputline,"maxiter=%lf", &numbuffer) ){
				calcdata->maxiter = (int)numbuffer;
				continue;
			}		
			if( sscanf(inputline,"printiter=%lf", &numbuffer) ){
				calcdata->printiter = (int)numbuffer;
				continue;
			}		
		}
	}
	fclose(fp);
	if(calcdata->geometrymode == AXIAL) {
		if( vip!=20 ) {
			printf("Imperfect inputfile. \n Program terminated. \n");
			return 12;
		}
	} else {
		if(calcdata->geometrymode == DESCARTES) {
			if( vip!=19 ) {
				printf("Imperfect inputfile. \n Program terminated. \n");
				return 12;
			}
		}

	}
	return 0;
}


/*******  foton data processing routines  *************************************************************************************************/



int insertdigifoto ( long int index, int multiscatter, tdata *calcdata ) {
	tdigifoto *temp;
	tdigifoto *kukker;
	
	calcdata->dfpointer = calcdata->dflisthead;
	if ( calcdata->dfpointer ) {
		calcdata->dfpointer = calcdata->dflisthead;

		if( calcdata->dfpointer->index > index){
			if( !(calcdata->dflisthead = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
				printf("Memory allocation error. \n Program terminated. \n");
				return 201;
			}
			calcdata->digifotonum++;
			calcdata->dflisthead->index = index;
			calcdata->dflisthead->n = 1;
			calcdata->dflisthead->multiscatter = multiscatter;
			calcdata->dflisthead->next = calcdata->dfpointer;
		} else {
			while( (calcdata->dfpointer) && (calcdata->dfpointer->index<index) ){
				kukker = calcdata->dfpointer;
				calcdata->dfpointer = calcdata->dfpointer->next;
			}
			if(calcdata->dfpointer) {
				if(calcdata->dfpointer->index == index) {
					calcdata->dfpointer->n++;
				} else {
					if( !(temp = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
						printf("Memory allocation error. \n Program terminated. \n");
						return 202;
					}
					calcdata->digifotonum++;
					temp->index = index;
					temp->multiscatter = multiscatter;
					temp->n = 1;
					kukker->next = temp;
					temp->next = calcdata->dfpointer;
				}
			} else {
				if( !(temp = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
					printf("Memory allocation error. \n Program terminated. \n");
					return 203 ;
				}
				calcdata->digifotonum++;
				temp->index = index;
				temp->multiscatter = multiscatter;
				temp->n = 1;
				kukker->next = temp;
				temp->next = calcdata->dfpointer;				
			}
		}
	} else {
		if( !(calcdata->dflisthead = (tdigifoto*)malloc(sizeof(tdigifoto)) )) {
			printf("Memory allocation error. \n Program terminated. \n");
			return 204;
		}
		calcdata->digifotonum++;
		calcdata->dflisthead->index = index;
		calcdata->dflisthead->multiscatter = multiscatter;
		calcdata->dflisthead->n = 1;
		calcdata->dflisthead->next = NULL;
	}
	return 0;
}


void readtwolines ( FILE *fp, tinputrecord *inputrecord, int firstevent ) {
	int a, firstlinelength;
	if( firstevent ){
		firstlinelength = 6;
	} else {
		firstlinelength = 7;
	}
	for( a=0; a<firstlinelength && !feof(fp); a++ ){
		fscanf(fp, "%d", &(inputrecord->firstline[a]) );
	}
	for( a=0; a<9 && !feof(fp); a++ ){
		fscanf(fp, "%le", &(inputrecord->secondline[a]) );
	}
}


long int digitaxial( double xd, double yd, double xu, double yu, double zu, double denergy, tdata *calcdata ){
	int maxxd, maxyd, maxphiu, maxzu;
	int offsxd, offsyd;
	int e, u, d;
	int uphii, uzi, dxi, dyi;
	double phiu;
	long int index;

	dxi=floor(xd/calcdata->pixd);
	dyi=floor(yd/calcdata->pixd);

	phiu = atan2(yu,xu);
	if ( phiu<0 ) phiu += 2*PI;
	uphii = floor( phiu/calcdata->pixphi );
	uzi = floor( (zu-calcdata->thick-calcdata->dist2)/calcdata->pixu );

	maxxd = round( calcdata->dx/calcdata->pixd );
	maxyd = round( calcdata->dy/calcdata->pixd );
	offsxd = round( maxxd / 2.0 );
	offsyd = round( maxyd / 2.0);
	d = round( (dxi+offsxd)+(dyi+offsyd)*maxxd );
	
	maxzu = round( calcdata->uz/calcdata->pixu );
	maxphiu = round( 2*PI/calcdata->pixphi );
	u = round( uphii + uzi*maxphiu );

	e = floor( denergy/calcdata->energy * round(100.0/calcdata->efelb) );
	
	index =	e+ 
		u*round(100.0/calcdata->efelb)+
		d*round( maxzu*maxphiu ) * round(100.0/calcdata->efelb);
	return index;
}

long int digit( double xd, double yd, double xu, double yu, double denergy, tdata *calcdata ){
	int maxxd, maxyd, maxxu, maxyu;
	int offsxd, offsyd, offsxu, offsyu;
	int e, u, d;
	int uxi, uyi, dxi, dyi;
	long int index;

	uxi=floor(xu/calcdata->pixu);
	uyi=floor(yu/calcdata->pixu);

	dxi=floor(xd/calcdata->pixd);
	dyi=floor(yd/calcdata->pixd);

	maxxd = round( calcdata->dx/calcdata->pixd );
	maxyd = round( calcdata->dy/calcdata->pixd );
	offsxd = round( maxxd / 2.0 );
	offsyd = round( maxyd / 2.0);
	d = round( (dxi+offsxd)+(dyi+offsyd)*maxxd );

	maxxu = round( calcdata->ux/calcdata->pixu );
	maxyu = round( calcdata->uy/calcdata->pixu );
	offsxu = round( maxxu / 2.0 );
	offsyu = round( maxyu / 2.0 );
	u = round( (uxi+offsxu)+(uyi+offsyu)*maxxu);

	e = floor( denergy/calcdata->energy * round(100.0/calcdata->efelb) );
	
	index =	e+ 
		u*round(100.0/calcdata->efelb)+
		d*round( maxxu*maxyu ) * round(100.0/calcdata->efelb);
	return index;
}


int readfotons ( tdata *calcdata ) {
	FILE * fp;
	double sum, lastscatenergy;
	char s[255];
	//int i1,i2,i3,i4;
	double xu, yu, zu;
	double xd, yd, zd;
	long int index;
	int errorcode;
	int multiscatter;
	tinputrecord inputrecord1, inputrecord2, inputrecord3;
		
	fp = fopen(calcdata->ptracfname, "rt");
	if( fp==NULL ) {
		printf("Error opening file: %s \n",calcdata->ptracfname);
		return 20;
	}

	do {
		fscanf( fp, "%s", s );
	} while( strcmp(s,"1000") && !feof(fp) );
	if(calcdata->ptraccellfilter) fscanf( fp, "%s", s );
	while( !feof(fp) ) {
		sum = 0;
		multiscatter = 0;
		readtwolines( fp, &inputrecord1, 1 );
		readtwolines( fp, &inputrecord2, 0 );
		if( !( inputrecord1.firstline[0] == 5000 && inputrecord2.firstline[0] == 9000 )){
			if( inputrecord2.secondline[2] < calcdata->thick) {
				while( (inputrecord1.secondline[6] == inputrecord2.secondline[6]) && inputrecord2.firstline[0] != 9000 && !feof(fp)){
					readtwolines( fp, &inputrecord2, 0 );
				}
				if ( inputrecord2.secondline[2]<calcdata->thick && inputrecord2.firstline[0] != 9000 && !feof(fp) ) {
					xd=inputrecord2.secondline[0];
					yd=inputrecord2.secondline[1];
					zd=inputrecord2.secondline[2];
					sum+=inputrecord1.secondline[6]-inputrecord2.secondline[6];
					lastscatenergy = inputrecord2.secondline[6];
					readtwolines( fp, &inputrecord3, 0 );
					while( inputrecord3.secondline[2] < calcdata->thick+calcdata->dist2 && inputrecord3.firstline[0] != 9000 && !feof(fp)){
						if( inputrecord3.secondline[6] < lastscatenergy ){
							sum+=lastscatenergy-inputrecord3.secondline[6];
							multiscatter = 1;
						}
						lastscatenergy = inputrecord3.secondline[6];
						readtwolines( fp, &inputrecord3, 0);
					}
					while(inputrecord3.secondline[6] == lastscatenergy && inputrecord3.firstline[0] != 9000 && !feof(fp)){
						readtwolines( fp, &inputrecord3, 0 );
					}
					if ( inputrecord3.secondline[6] < lastscatenergy && !feof(fp) ) {
						xu = inputrecord3.secondline[0];
						yu = inputrecord3.secondline[1];
						zu = inputrecord3.secondline[2];
						if(calcdata->geometrymode == DESCARTES ) {
							index = digit(xd, yd, xu, yu, sum, calcdata);
							
							if( fabs(xu) < calcdata->ux/2.0 &&
							    fabs(yu) < calcdata->uy/2.0 &&
							    fabs(xd) < calcdata->dx/2.0 &&
							    fabs(yd) < calcdata->dy/2.0	) {
								if( errorcode = insertdigifoto(index, multiscatter, calcdata) ) {
									printf("Insert failed. \n");
									return errorcode;
								}
							}
						} else {
							if(calcdata->geometrymode == AXIAL ) {
								index = digitaxial(xd, yd, xu, yu, zu, sum, calcdata);
								//printf("%lf, %lf, %lf, %lf, %lf ==> %ld\n", xd, yd, xu, yu, zu, index);
								if( zu < calcdata->thick+calcdata->dist2+calcdata->uz &&
								    zu > calcdata->thick+calcdata->dist2 &&
								    fabs(xd) < calcdata->dx/2.0 &&
								    fabs(yd) < calcdata->dy/2.0	) {
									if( errorcode = insertdigifoto(index, multiscatter, calcdata) ) {
										printf("Insert failed. \n");
										return errorcode;
									}
								}
							}

						}

					}
					while( inputrecord3.firstline[0] != 9000 && !feof(fp)) {
						readtwolines( fp, &inputrecord3, 0 );
					}
				} else {
					while( inputrecord2.firstline[0] != 9000 && !feof(fp)) {
						readtwolines( fp, &inputrecord2, 0 );
					}
				}
			} else {
				while( inputrecord2.firstline[0] != 9000 && !feof(fp)) {
					readtwolines( fp, &inputrecord2, 0 );
				}
			}
		
		}

		if( !feof(fp) ) {
			fscanf( fp, "%s", s );
			fscanf( fp, "%s", s );
			if(calcdata->ptraccellfilter) fscanf( fp, "%s", s );
		}
	}
	fclose(fp);
	return 0;
}


void filldfvect( tdata *calcdata ) {
	long int i;
	calcdata->dfpointer = calcdata->dflisthead;
	for( i=0; i<calcdata->digifotonum; i++ ){
		calcdata->dfvect[i].index = calcdata->dfpointer->index;
		calcdata->dfvect[i].multiscatter = calcdata->dfpointer->multiscatter;
		calcdata->dfvect[i].n = calcdata->dfpointer->n;
		calcdata->dfvect[i].nonzeroprob = 0;
		calcdata->dfpointer = calcdata->dfpointer->next;
	}
}


/*******  print result  *******************************************************************************************************************/


int yeah ( int n, tdata *calcdata ) {
	char outfn[256];
	FILE *fp;
	long int l;
	int x,y, gx,gy;

	printf("After %d. iteration: \n",n);
	sprintf(outfn,"%s.output%d", calcdata->ptracfname, n);
	printf("\tWriting results to: %s ... \n",outfn);
	if( !(fp=fopen(outfn,"wt")) ){
		printf("Error opening file: %s. \n Program terminated. \n",outfn);
		return 40;
	}
	
	gx = round(calcdata->mx / calcdata->pixm);
	gy = round(calcdata->my / calcdata->pixm);
	l = 0;
	for( y=0; y<gy; y++ ){
		for( x=0; x<gx; x++ ){
			fprintf(fp,"%.5E\t",calcdata->lambda[calcdata->flipflop][l]);
			l++;
		}
		fprintf(fp,"\n");
	}
	fflush(fp);
	fclose(fp);
	printf("done\n");
	return 0;
}


/*******  calculating probmx  *************************************************************************************************************/


double kleinnishina ( tdata *calcdata, double costheta, double outenergy, double rdu) {
	double gamma, amounte, rnull,  sintheta, dtheta, prob21, prob22;

	amounte=calcdata->rho/calcdata->M*AVOGADRO*calcdata->Z*(calcdata->pixd*calcdata->pixd*calcdata->thick);
	gamma=calcdata->energy/0.511;
	rnull=2.818*10E-13;
	sintheta=sqrt(1-costheta*costheta);
	dtheta=calcdata->pixu*(calcdata->dist2+calcdata->thick/2.0)/pow(rdu,2);
	prob21=2*PI*pow(rnull,2)*sintheta*dtheta*((1+pow(costheta,2))/2.0)*(1.0/pow((1+gamma*(1-costheta)),2));
	prob22=( pow(gamma,2)*pow( (1-costheta),2 ) ) / ( (1+pow(costheta,2)) * (1+gamma*(1-costheta)) );
	return((prob21*(1+prob22))*amounte);
}


/*double circellipse (tdata *calcdata, double costheta, double rpd, double alpha) {
	double sintheta, tgtheta, theta, r, a1, a2, a, b;

	sintheta=sqrt(1-pow(costheta,2));
	tgtheta=sintheta/costheta;
	theta=acos(costheta);
	r=rpd*tgtheta;
	a1=r*costheta/sin(theta+(PI/2.0-alpha));
	a2=r*costheta/sin((PI/2.0-alpha)-theta);
	if(0<a1 && 0<a2) {
		a=(a1+a2)/2.0;
		b=sqrt(fabs(  	pow(((a-a1)*cos(PI/2.0-alpha)),2) + 
				pow((a-a1),2) + 
				pow(((rpd+(a-a1)*cos(PI/2.0-alpha))*tgtheta),2)   ));
		
		return(PI*(1.5*(a+b)-sqrt(a*b)));
	} else {
		return 0;
	}

}*/


void jimithesecond ( tdata *calcdata ) {	/* in memoriam Jimi Hendrix */
	double xu, yu, zu;
	double ru, phiu;
	double xd, yd, zd;
	double xm, ym, zm;
	double rmd, smd, rdu;
	double v1x, v1y, v1z;
	double v2x, v2y, v2z;
	double alpha, costheta, sintheta;
	double outenergy, scatenergy;
	double reff;
	
	double prob, prob1, prob2, prob3;
	
	long int col,row;
	long int index;
	
	int numiu, numju, maxiu, maxju;
	int numid, numjd, maxid, maxjd;
	int numim, numjm, maxim, maxjm;

	maxiu = round(2*PI / calcdata->pixphi);
	maxju = round(calcdata->uz / calcdata->pixu);
	maxid = round(calcdata->dx / calcdata->pixd);
	maxjd = round(calcdata->dy / calcdata->pixd);
	maxim = round(calcdata->mx / calcdata->pixm);
	maxjm = round(calcdata->my / calcdata->pixm);

	ru = calcdata->ur;
	
	reff = sqrt( calcdata->pixu * calcdata->pixphi * calcdata->ur / PI );
	
	for( numjm = 0; numjm<maxjm; numjm++ ) {
		for(numim = 0; numim<maxim; numim++ ) {

			col=round(numim+numjm*(calcdata->mx/calcdata->pixm));
			row = 0;
			calcdata->colsum[col] = 0;
						
			xm=-(calcdata->mx/2.0)+(numim*calcdata->pixm+(calcdata->pixm/2.0));
			ym=-(calcdata->my/2.0)+(numjm*calcdata->pixm+(calcdata->pixm/2.0));
			zm=calcdata->dist1*(-1);

			for( numjd = 0; numjd<maxjd; numjd++ ) {
				for(numid = 0; numid<maxid; numid++ ) {

					xd=-(calcdata->dx/2.0)+(numid*calcdata->pixd+(calcdata->pixd/2.0));
					yd=-(calcdata->dy/2.0)+(numjd*calcdata->pixd+(calcdata->pixd/2.0));
					zd=calcdata->thick/2.0;
					rmd=sqrt(pow((xm-xd),2)+pow((ym-yd),2)+pow(zm-zd,2));
					smd=sqrt(pow((xm-xd),2)+pow((ym-yd),2));
					alpha=asin(smd/rmd);
					prob1=((pow(calcdata->pixd,2)/cos(alpha))/(pow(rmd,2)*4*PI));

					v1x=(xd-xm)/rmd;
					v1y=(yd-ym)/rmd;
					v1z=(zd-zm)/rmd;
					
					for( numju = 0; numju<maxju; numju++ ) {
						for(numiu = 0; numiu<maxiu; numiu++ ) {
							
							phiu = calcdata->pixphi*(numiu+0.5);
							zu= calcdata->thick+calcdata->dist2+calcdata->pixu*(numju+0.5);
							xu=ru*cos(phiu);
							yu=ru*sin(phiu);
							rdu=sqrt(pow(xu-xd,2)+pow(yu-yd,2)+pow(zu-zd,2));
							v2x=(xu-xd)/rdu;
							v2y=(yu-yd)/rdu;
							v2z=(zu-zd)/rdu;
							costheta=v1x*v2x+v1y*v2y+v1z*v2z;
							sintheta=sqrt(1-costheta*costheta);
							outenergy=1.0/((1.0/calcdata->energy)+(1-costheta)/0.511);
							scatenergy=calcdata->energy-outenergy;
							prob2=kleinnishina(calcdata, costheta, outenergy, rdu);

	
							if( (rdu*PI*sintheta)/reff>1 ) prob3=reff/(rdu*PI*sintheta);
							else prob3 = 1;
								
							index =	floor(scatenergy/calcdata->energy * 100.0/calcdata->efelb) + 
								round(100.0/calcdata->efelb)* (numju*maxiu+numiu) +
								round(100.0/calcdata->efelb)* maxiu*maxju * (numjd*maxid+numid);
							prob = prob1*prob2*prob3;
							calcdata->colsum[col] += prob;
							if( row < calcdata->digifotonum && prob ) {
								while( row < calcdata->digifotonum && calcdata->dfvect[row].index < index){
									row++;
								}
								if( row < calcdata->digifotonum && calcdata->dfvect[row].index == index ) {
									calcdata->probmx[row][col] = prob;
									calcdata->dfvect[row].nonzeroprob = 1;
									row++;
								}
							}
						}
					}			
				}
			}			
		}
	}
}





void returnofjimi ( tdata *calcdata ) {	/* in memoriam Jimi Hendrix */
	double xu, yu, zu;
	double xd, yd, zd;
	double xm, ym, zm;
	double rmd, smd, rdu;
	double v1x, v1y, v1z;
	double v2x, v2y, v2z;
	//double pux, puy, puz;
	double alpha, costheta, sintheta;
	double outenergy, scatenergy;
	double reff;
	
	double prob, prob1, prob2, prob3;
	
	long int col,row;
	long int index;
	
	int numiu, numju, maxiu, maxju;
	int numid, numjd, maxid, maxjd;
	int numim, numjm, maxim, maxjm;

	maxiu = round(calcdata->ux / calcdata->pixu);
	maxju = round(calcdata->uy / calcdata->pixu);
	maxid = round(calcdata->dx / calcdata->pixd);
	maxjd = round(calcdata->dy / calcdata->pixd);
	maxim = round(calcdata->mx / calcdata->pixm);
	maxjm = round(calcdata->my / calcdata->pixm);
	zu=calcdata->dist2+calcdata->thick;
	
	reff =  calcdata->pixu / sqrt(PI);

	for( numjm = 0; numjm<maxjm; numjm++ ) {
		for(numim = 0; numim<maxim; numim++ ) {

			col=round(numim+numjm*(calcdata->mx/calcdata->pixm));
			row = 0;
			calcdata->colsum[col] = 0;
						
			
			xm=-(calcdata->mx/2.0)+(numim*calcdata->pixm+(calcdata->pixm/2.0));
			ym=-(calcdata->my/2.0)+(numjm*calcdata->pixm+(calcdata->pixm/2.0));
			zm=calcdata->dist1*(-1);

			for( numjd = 0; numjd<maxjd; numjd++ ) {
				for(numid = 0; numid<maxid; numid++ ) {

					xd=-(calcdata->dx/2.0)+(numid*calcdata->pixd+(calcdata->pixd/2.0));
					yd=-(calcdata->dy/2.0)+(numjd*calcdata->pixd+(calcdata->pixd/2.0));
					zd=calcdata->thick/2.0;
					rmd=sqrt(pow((xm-xd),2)+pow((ym-yd),2)+pow(zm-zd,2));
					smd=sqrt(pow((xm-xd),2)+pow((ym-yd),2));
					alpha=asin(smd/rmd);
					prob1=((pow(calcdata->pixd,2)/cos(alpha))/(pow(rmd,2)*4*PI));

					v1x=(xd-xm)/rmd;
					v1y=(yd-ym)/rmd;
					v1z=(zd-zm)/rmd;

					for( numju = 0; numju<maxju; numju++ ) {
						for(numiu = 0; numiu<maxiu; numiu++ ) {
	
							
							
							xu=-(calcdata->ux/2.0)+(numiu*calcdata->pixu+(calcdata->pixu/2.0));
							yu=-(calcdata->uy/2.0)+(numju*calcdata->pixu+(calcdata->pixu/2.0));
							rdu=sqrt(pow(xu-xd,2)+pow(yu-yd,2)+pow(zu-zd,2));
							v2x=(xu-xd)/rdu;
							v2y=(yu-yd)/rdu;
							v2z=(zu-zd)/rdu;
							costheta=v1x*v2x+v1y*v2y+v1z*v2z;
							sintheta = sqrt(1-costheta*costheta);
							outenergy=1.0/((1.0/calcdata->energy)+(1-costheta)/0.511);
							scatenergy=calcdata->energy-outenergy;
							prob2=kleinnishina(calcdata, costheta, outenergy, rdu);

							if( (rdu*PI*sintheta)/reff>1 ) prob3=reff/(rdu*PI*sintheta);
							else prob3 = 1;

							index =	floor(scatenergy/calcdata->energy * 100.0/calcdata->efelb) + 
								round(100.0/calcdata->efelb)* (numju*maxiu+numiu) +
								round(100.0/calcdata->efelb)* maxiu*maxju * (numjd*maxid+numid);
							prob = prob1*prob2*prob3;
							calcdata->colsum[col] += prob;
							if( row < calcdata->digifotonum && prob ) {
								while( row < calcdata->digifotonum && calcdata->dfvect[row].index < index){
									row++;
								}
								if( row < calcdata->digifotonum && calcdata->dfvect[row].index == index ) {
									calcdata->probmx[row][col] = prob;
									calcdata->dfvect[row].nonzeroprob = 1;
									row++;
								}
							}
						}
					}			
				}
			}			
		}
	}
}


/*******  let's rock!  ********************************************************************************************************************/


void imagegeneration ( tdata *calcdata ) {
	int n;
	long int j,i,k;
	int flip,flop;
	double basszus, violin;
	
	for( n=1; n<=calcdata->maxiter; n++){
		flip = (n+1) % 2;
		flop = n % 2;
			
		for( j=0; j<calcdata->pmxcolnum; j++ ){
			basszus = 0;

			for( i=0; i<calcdata->digifotonum; i++) {
				if(calcdata->dfvect[i].nonzeroprob){
					violin = 0;
					for( k=0; k<calcdata->pmxcolnum; k++ ) {
						violin += calcdata->probmx[i][k] * calcdata->lambda[flop][k];
					}
					if (violin) basszus += calcdata->probmx[i][j] * calcdata->dfvect[i].n / violin;
				}
			}			
			calcdata->lambda[flip][j] = calcdata->lambda[flop][j] * (1.0/calcdata->colsum[j]) * basszus;
		}
		calcdata->flipflop = flip;
		if( n%calcdata->printiter == 0 ) yeah( n, calcdata );				
	}		
}


/*******  main  ***************************************************************************************************************************/


int main( int argc, char *argv[] ) {
	int errorcode;
	tdata calcdata; 						/* static structure, contains all data */
	long int jajj,jujj,jajjjujj,i;
	
	printf("\n\n============================================================\n");
	printf("\n\n\t  Compton Camera image generator software.\n");	/* Welcome message */
	printf("\n\n\t\t version:\n\t\t\t 0.18 beta\n");
	printf("\t\t date:\n\t\t\t 8 November 2006\n");
	printf("\t\t author:\n\t\t\t Karoly Nagy\n\t\t\t Szilard Szalay\n");
	printf("\n\n============================================================\n");
	printf("\n\n");

	init ( &calcdata );						/* initializing datastructure I. - pointers */
	
	if(argc == 2){							/* need only one argument */
		strcpy(calcdata.infname, argv[1]);			/* which is the name of the inputile */
	} else {
		printf("Usage: %s <inputfile> \n", argv[0]);
		return 1;
	}

	printf("\nReading inputfile ... \n");
	if( errorcode = readinput ( &calcdata ) ) return errorcode;	/* reads the inputfile */
	printf("done. \n");


	printf("Geometry: ");
	if( calcdata.geometrymode == DESCARTES ) printf("stanrard\n");
	else if( calcdata.geometrymode == AXIAL ) printf("axial\n");
	else if( calcdata.geometrymode == POLAR ) { printf("polar\n");
		printf(" khmm... it has not been implemented yet. \n Sorry. Program terminated.\n");
		return -1;
	}

	
	
	printf("\nReading fotons from file: %s ... \n", calcdata.ptracfname);
	if( errorcode = readfotons ( &calcdata ) ) return errorcode;	/* reads foton data from MCNP outputfile */
	printf("done. \n");
	printf("Number of elements: %ld\n",calcdata.digifotonum);

	printf("\nInitializing dfvect ... \n");
	if( errorcode = initdfvect ( &calcdata ) ) return errorcode;	/* initializing datastructure II. - fotovectorn */
	printf("done. \n");

	printf("\nWriting fotons to dfvect ... \n");
	filldfvect ( &calcdata );					/* writing fotons to dfvect */
	printf("done. \n");

	freedflist( &calcdata ); 

	printf("\nInitializing pmatrix ... \n");
	if( errorcode = initpm ( &calcdata ) ) return errorcode;	/* initializing datastructure III. - probability matrix */
	printf("done. \n");						/* it needs the geometry and foton data */
	printf("Dimensions: %ld x %ld \n", calcdata.pmxcolnum, calcdata.pmxrownum);
	
	printf("\nLoading pmatrix ... \n");
	if( calcdata.geometrymode == DESCARTES ) returnofjimi ( & calcdata );	/* calculates the probability matrix elements */
	if( calcdata.geometrymode == AXIAL ) jimithesecond ( & calcdata );
	printf("done. \n");
	printf("number of elements = %ld x %ld \n", calcdata.pmxcolnum, (long int)round(calcdata.pmxrownum*calcdata.efelb/100));



	jajj = 0;
	jujj = 0;
	jajjjujj = 0;
	for( i=0; i<calcdata.digifotonum; i++ ) {
		if(!calcdata.dfvect[i].nonzeroprob) jajj++;
		if(calcdata.dfvect[i].multiscatter) jujj++;
		if(!calcdata.dfvect[i].nonzeroprob && calcdata.dfvect[i].multiscatter) jajjjujj++;
	}
	printf("0valsegu                 foton : %ld\n",jajj);
	printf("Multiscatter             foton : %ld\n",jujj);
	printf("0valsegu && Multiscatter foton : %ld\n",jajjjujj);
	




	printf("\nInitializing lambda-list ... \n");
	if( errorcode = initlambda ( &calcdata ) ) return errorcode;	/* initializing datastructure IV. - lambda vector */
	printf("done. \n");
			
	printf("\nComputing started.\n");
	imagegeneration ( & calcdata );					/* iterative image generating algorythm */
	printf("computing done.\n\n");
	
	return 0;
}


/*
   
(. Y .)

*/

